#!/usr/bin/python

################################################################################
##3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 
##
## Info:
##    gtk+-based UI-frontend for Geek3D's MadShaders
##
## Run:
##    chmod +x mad-shaders-frontend.py
##    ./mad-shaders-frontend.py
##
## Copyright 2014 Mirco Mueller
##
## Author:
##    Mirco "MacSlow" Mueller <macslow@gmail.com>
##
## This program is free software: you can redistribute it and/or modify it
## under the terms of the GNU General Public License version 3, as published
## by the Free Software Foundation.
##
## This program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranties of
## MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License along
## with this program.  If not, see <http://www.gnu.org/licenses/>.
##
################################################################################

import os
from gi.repository import Gtk

OPT_FULLSCREEN = ' /fullscreen'
OPT_BENCHMARK  = ' /madshaders_benchmark'
OPT_WIDTH      = ' /width='
OPT_HEIGHT     = ' /height='
OPT_DEMO       = ' /demofile='
DEMO_CMD       = './GLSLHacker /no_menubar /disable_tcpip_server'

class FrontendUI:
	def onDemosComboBoxChanged (self, widget, data=None):
		iter = widget.get_active_iter ()
		if iter != None:
			model = widget.get_model ()
			entry, desc, url, demo = model[iter][:4]
			self.labelDescription.set_text (desc)
			self.linkHomepage.set_uri (url)
			self.demo = "\\\"" + demo + "\\\""

	def onCheckbuttonFullscreenToggled (self, widget, data=None):
		if (widget.get_active ()):
			self.fullscreen = OPT_FULLSCREEN
		else:
			self.fullscreen = ""

	def onResolutionComboBoxChanged (self, widget, data=None):
		iter = widget.get_active_iter ()
		if iter != None:
			model = widget.get_model ()
			entry, name = model[iter][:2]
			if name == "1":
				self.width = "800"
				self.height = "600"
			if name == "2":
				self.width = "1024"
				self.height = "640"
			if name == "3":
				self.width = "1280"
				self.height = "720"
			if name == "4":
				self.width = "1920"
				self.height = "1080"

	def onButtonDemoClicked (self, widget, data=None):
		os.system (DEMO_CMD +
				   OPT_DEMO + self.demo +
				   OPT_WIDTH + self.width +
				   OPT_HEIGHT + self.height +
				   self.fullscreen)

	def onButtonBenchmarkClicked (self, widget, data=None):
		os.system (DEMO_CMD + OPT_BENCHMARK +
				   OPT_DEMO + self.demo +
				   OPT_WIDTH + self.width +
				   OPT_HEIGHT + self.height +
				   self.fullscreen)

	def __init__(self):
		# setup/load glade UI-description filie
		self.builder = Gtk.Builder ()
		self.builder.add_from_file ("./mad-shaders-frontend.glade")

		handlers = {
			"onDemosComboBoxChanged": self.onDemosComboBoxChanged,
			"onCheckbuttonFullscreenToggled": self.onCheckbuttonFullscreenToggled,
			"onResolutionComboBoxChanged": self.onResolutionComboBoxChanged,
			"onButtonDemoClicked": self.onButtonDemoClicked,
			"onButtonBenchmarkClicked": self.onButtonBenchmarkClicked,
			"onMainWindowDestroy":  Gtk.main_quit
		}
		self.builder.connect_signals(handlers)
		self.demo = "\\\"demos/hot_shower_gl2.xml\\\""
		self.fullscreen = ""
		self.width = "800"
		self.height = "600"

		self.mainWindow = self.builder.get_object("mainWindow")
		self.demosComboBox = self.builder.get_object("demosComboBox")
		self.demosComboBox = self.builder.get_object("demosComboBox")
		self.labelDescription = self.builder.get_object("labelDescription")
		self.labelDescription.set_text ("FurMark GPU burner (www.ozone3d.net/benchmarks/fur/)")
		self.linkHomepage = self.builder.get_object("linkHomepage")
		self.labelInfo = self.builder.get_object("labelInfo")

		self.mainWindow.set_title ("Geeks3D MadShaders 0.3.0")
		self.mainWindow.set_icon_from_file ("mad-shaders-frontend-icon.png")
		self.mainWindow.show_all ()
		self.linkHomepage.set_uri ("https://www.shadertoy.com/view/4df3Wj?from=geeks3dmadshaders")
		self.linkHomepage.set_label ("visit Demo's Homepage")
		self.labelInfo.set_label ("MadShaders- Frontend (C) 2014 Mirco \"MacSlow\" Mueller")

		store = Gtk.ListStore (str, str, str, str)
		store.append (["Hot Shower", "Shadertoy (www.shadertoy.com) - Author: Kali - Tags: fractal, particles", "https://www.shadertoy.com/view/4df3Wj?from=geeks3dmadshaders", "demos/hot_shower_gl2.xml"])
		store.append (["Generators Redux", "Shadertoy (www.shadertoy.com) - Author: Kali - Tags: 3d, raytracing, raymarching, fractal, distancefield, tv", "https://www.shadertoy.com/view/lsXGWl?from=geeks3dmadshaders", "demos/generator_redux_gl2.xml"])
		store.append (["Relentless", "Shadertoy (www.shadertoy.com) - Author: srtuss - Tags: rings, scifi", "https://www.shadertoy.com/view/lss3WS?from=geeks3dmadshaders", "demos/relentless_gl2.xml"])
		store.append (["Balls and occlusion", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: 3d, raytracing, occlusion", "https://www.shadertoy.com/view/ldX3Ws?from=geeks3dmadshaders", "demos/balls_and_occlusion_gl2.xml"])
		store.append (["Basic fractal", "Shadertoy (www.shadertoy.com) - Author: PauloFalcao - Tags: 2d, fractal", "https://www.shadertoy.com/view/Mss3Wf?from=geeks3dmadshaders", "demos/basic_fractal_gl2.xml"])
		store.append (["The inversion machine", "Shadertoy (www.shadertoy.com) - Author: Kali - Tags: distancefield, raymarch, sphereinversion", "https://www.shadertoy.com/view/4dsGD7?from=geeks3dmadshaders", "demos/the_inversion_machine_gl2.xml"])
		store.append (["Electron", "Shadertoy (www.shadertoy.com) - Author: gopher - Tags: raycasting, perlinnoise, fbm", "https://www.shadertoy.com/view/MslGRn?from=geeks3dmadshaders", "demos/electron_gl2.xml"])
		store.append (["Elevated", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, raymarching", "https://www.shadertoy.com/view/MdX3Rr?from=geeks3dmadshaders", "demos/elevated_gl2.xml"])
		store.append (["Perlin noise sphere", "Shadertoy (www.shadertoy.com) - Author: triggerHLM - Tags: perlinnoise, spheretracing, distortedsphere", "https://www.shadertoy.com/view/Msf3DH?from=geeks3dmadshaders", "demos/perlin_noise_sphere_gl2.xml"])
		store.append (["Furball", "Shadertoy (www.shadertoy.com) - Author: simesgreen - Tags: raymarch, hair, fur, fluffy", "https://www.shadertoy.com/view/XsfGWN?from=geeks3dmadshaders", "demos/furball_gl2.xml"])
		store.append (["Voxel edges", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, raymarching, voxel, occlusion, dda", "https://www.shadertoy.com/view/4dfGzs?from=geeks3dmadshaders", "demos/voxel_edges_gl2.xml"])
		store.append (["Cell", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, distancefield, sound, reactive, music, raymarching", "https://www.shadertoy.com/view/Xdl3R4?from=geeks3dmadshaders", "demos/cell_gl2.xml"])
		store.append (["Angels", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, raymarching, distancefield", "https://www.shadertoy.com/view/lssGRM?from=geeks3dmadshaders", "demos/angels_gl2.xml"])
		store.append (["Insect", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, distancefield, raymarching, animation", "https://www.shadertoy.com/view/Mss3zM?from=geeks3dmadshaders", "demos/insect_gl2.xml"])
		store.append (["Piano", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, raymarching, distancefield", "https://www.shadertoy.com/view/ldl3zN?from=geeks3dmadshaders", "demos/piano_gl2.xml"])
		store.append (["Truchet Tentacles", "Shadertoy (www.shadertoy.com) - Author: WAHa_06x36 - Tags: raymarching, distancefield, truchet", "https://www.shadertoy.com/view/ldfGWn?from=geeks3dmadshaders", "demos/truchet_tentacles_gl2.xml"])
		store.append (["Julia", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: 2d, fractal", "https://www.shadertoy.com/view/4dfGRn?from=geeks3dmadshaders", "demos/julia_gl2.xml"])
		store.append (["Main Sequence Star", "Shadertoy (www.shadertoy.com) - Author: flight404 - Tags: 3dnoise, perlin, star, corona", "https://www.shadertoy.com/view/4dXGR4?from=geeks3dmadshaders", "demos/main_sequence_star_gl2.xml"])
		store.append (["Flame", "Shadertoy (www.shadertoy.com) - Author: XT95 - Tags: 3d, distancefield, transparency", "https://www.shadertoy.com/view/MdX3zr?from=geeks3dmadshaders", "demos/flame_gl2.xml"])
		store.append (["Fruxis", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, raymarching, distancefields", "https://www.shadertoy.com/view/ldl3zl?from=geeks3dmadshaders", "demos/fruxis_gl2.xml"])
		store.append (["Glass Eel", "Shadertoy (www.shadertoy.com) - Author: Kali - Tags: raymarch, refraction", "https://www.shadertoy.com/view/MsfGWS?from=geeks3dmadshaders", "demos/glass_eel_gl2.xml"])
		store.append (["Ball of fire", "Shadertoy (www.shadertoy.com) - Author: Trisomie21 - Tags: 3dnoise, perlin", "https://www.shadertoy.com/view/lsf3RH?from=geeks3dmadshaders", "demos/ball_of_fire_gl2.xml"])
		store.append (["Distance Estimated DoF", "Shadertoy (www.shadertoy.com) - Author: eiffie - Tags: dof, fakedof", "https://www.shadertoy.com/view/MsBGRh?from=geeks3dmadshaders", "demos/de_dof_gl2.xml"])
		store.append (["Fractraps III", "Shadertoy (www.shadertoy.com) - Author: Kali - Tags: fractal, orbittraps", "https://www.shadertoy.com/view/XsB3R1?from=geeks3dmadshaders", "demos/fractraps3_gl2.xml"])
		store.append (["Sierpkinki - 3D", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: 3d, raymarching, fractal, distancefield, sierpinksi", "https://www.shadertoy.com/view/4dl3Wl?from=geeks3dmadshaders", "demos/sierpkinki_3d_gl2.xml"])
		store.append (["Screen space bump mapping", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: 3d, bump, bumpmapping, mikkelsen", "https://www.shadertoy.com/view/ldSGzR?from=geeks3dmadshaders", "demos/screen_space_bump_mapping_gl2.xml"])
		store.append (["Britney's Spaceship", "Shadertoy (www.shadertoy.com) - Author: Kali - Tags: 3d, raymarching, fractal, amazingsurface", "https://www.shadertoy.com/view/4sj3Rh?from=geeks3dmadshaders", "demos/britney_spaceship_gl2.xml"])
		store.append (["Carnival Ride", "Shadertoy (www.shadertoy.com) - Author: eiffie - Tags: shadows, reflections, fuzzy", "https://www.shadertoy.com/view/Xd2GR1?from=geeks3dmadshaders", "demos/carnival_ride_gl2.xml"])
		store.append (["Fractal Cartoon", "Shadertoy (www.shadertoy.com) - Author: Kali - Tags: edgedetection", "https://www.shadertoy.com/view/4djGz1?from=geeks3dmadshaders", "demos/fractal_cartoon_gl2.xml"])
		store.append (["Spiral tiling", "Shadertoy (www.shadertoy.com) - Author: knighty - Tags: 3d, sdf, spiral", "https://www.shadertoy.com/view/ls2GRz?from=geeks3dmadshaders", "demos/spiral_tiling_gl2.xml"])
		store.append (["Storm", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: 3d, raymarching", "https://www.shadertoy.com/view/Xd23zh?from=geeks3dmadshaders", "demos/storm_gl2.xml"])
		store.append (["Mars Flight", "Shadertoy (www.shadertoy.com) - Author: Dave_Hoskins - Tags: spheretracing, mars, marsflight", "https://www.shadertoy.com/view/Md23Wz?from=geeks3dmadshaders", "demos/mars_flight_gl2.xml"])
		store.append (["Rounded Voxels", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, raymarching, distancefield, voxel", "https://www.shadertoy.com/view/4djGWR?from=geeks3dmadshaders", "demos/rounded_voxels_gl2.xml"])
		store.append (["Dodecahedron", "Shadertoy (www.shadertoy.com) - Author: gltracy - Tags: raymarching, ao, dodecahedron", "https://www.shadertoy.com/view/MdS3Rw?from=geeks3dmadshaders", "demos/dodecahedron_gl2.xml"])
		store.append (["Polar", "Shadertoy (www.shadertoy.com) - Author: gltracy - Tags: math, polar", "https://www.shadertoy.com/view/4sSGDR?from=geeks3dmadshaders", "demos/polar_gl2.xml"])
		store.append (["Prairie", "Shadertoy (www.shadertoy.com) - Author: eiffie - Tags: clouds, grass, soc", "https://www.shadertoy.com/view/Ms23zw?from=geeks3dmadshaders", "demos/prairie_gl2.xml"])
		store.append (["MagicaVoxel", "Shadertoy (www.shadertoy.com) - Author: gltracy - Tags: shadow, voxel, edge, ao", "https://www.shadertoy.com/view/MdBGRm?from=geeks3dmadshaders", "demos/magica_voxel_gl2.xml"])
		store.append (["Space Racing Lite", "Shadertoy (www.shadertoy.com) - Author: Kali - Tags: fractal, raymarch, space, race", "https://www.shadertoy.com/view/XsBGzw?from=geeks3dmadshaders", "demos/space_racing_lite_gl2.xml"])
		store.append (["Buoy", "Shadertoy (www.shadertoy.com) - Author: TekF - Tags: raytracing, raymarching, noise, reflection, refraction", "https://www.shadertoy.com/view/XdsGDB?from=geeks3dmadshaders", "demos/buoy_gl2.xml"])
		store.append (["Eye of Sauron", "Shadertoy (www.shadertoy.com) - Author: Dave_Hoskins - Tags: 3d, raytracing, fractal, eyeofsauron", "https://www.shadertoy.com/view/ldBGWW?from=geeks3dmadshaders", "demos/eye_of_sauron_gl2.xml"])
		store.append (["Misty Lake", "Shadertoy (www.shadertoy.com) - Author: reinder - Tags: raymarching, reflection, clouds, refraction, water, fog, mountains, lake", "https://www.shadertoy.com/view/MsB3WR?from=geeks3dmadshaders", "demos/misty_lake_gl2.xml"])
		store.append (["Marble", "Shadertoy (www.shadertoy.com) - Author: mmalex - Tags: raymarching, glass, marble", "https://www.shadertoy.com/view/XsSGDh?from=geeks3dmadshaders", "demos/marble_gl2.xml"])
		store.append (["Flaring", "Shadertoy (www.shadertoy.com) - Author: nimitz - Tags: procedural, 2d, flare", "https://www.shadertoy.com/view/lsSGzy?from=geeks3dmadshaders", "demos/flaring_gl2.xml"])
		store.append (["Flames", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, raymarching, volumetric", "https://www.shadertoy.com/view/MsXGRf?from=geeks3dmadshaders", "demos/flames_gl2.xml"])
		store.append (["Chain Reaction", "Shadertoy (www.shadertoy.com) - Author: eiffie - Tags: reflection, soc", "https://www.shadertoy.com/view/4sB3Dm?from=geeks3dmadshaders", "demos/chain_reaction_gl2.xml"])
		store.append (["Dolphin", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, raymarching, distancefield", "https://www.shadertoy.com/view/4sS3zG?from=geeks3dmadshaders", "demos/dolphin_gl2.xml"])
		store.append (["Fish Swimming", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, raymarching, distancefield, fish", "https://www.shadertoy.com/view/ldj3Dm?from=geeks3dmadshaders", "demos/fish_swimming_gl2.xml"])
		store.append (["Creepy Mountains", "Shadertoy (www.shadertoy.com) - Author: Kali - Tags: 2d, volumetric", "https://www.shadertoy.com/view/4sSGWm?from=geeks3dmadshaders", "demos/creepy_mountains_gl2.xml"])
		store.append (["Circuits", "Shadertoy (www.shadertoy.com) - Author: Kali - Tags: 2d, fractal, orbittraps", "https://www.shadertoy.com/view/4dSGDW?from=geeks3dmadshaders", "demos/circuits_gl2.xml"])
		store.append (["Grid of Cylinders", "Shadertoy (www.shadertoy.com) - Author: iq - Tags: procedural, 3d, raytracing, grid, cylinder, regulargrid", "https://www.shadertoy.com/view/4dSGW1?from=geeks3dmadshaders", "demos/grid_of_cylinders_gl2.xml"])
		store.append (["#2120.0", "GLSL Sandbox (glsl.heroku.com) - demo #2120.0", "http://glsl.heroku.com/e#2120.0", "demos/demo_2120_0_gl2.xml"])
		store.append (["#2902.4", "GLSL Sandbox (glsl.heroku.com) - demo #2902.4", "http://glsl.heroku.com/e#2902.4", "demos/demo_2902_4_gl2.xml"])
		store.append (["#3155.0", "GLSL Sandbox (glsl.heroku.com) - demo #3155.0", "http://glsl.heroku.com/e#3155.0", "demos/demo_3155_0_gl2.xml"])
		store.append (["#3281.0", "GLSL Sandbox (glsl.heroku.com) - demo #3281.0", "http://glsl.heroku.com/e#3281.0", "demos/demo_3281_0_gl2.xml"])
		store.append (["#6254.0", "GLSL Sandbox (glsl.heroku.com) - demo #6254.0", "http://glsl.heroku.com/e#6254.0", "demos/demo_6254_0_gl2.xml"])
		store.append (["#7424.0", "GLSL Sandbox (glsl.heroku.com) - demo #7424.0", "http://glsl.heroku.com/e#7424.0", "demos/demo_7424_0_gl2.xml"])
		store.append (["#7878.1", "GLSL Sandbox (glsl.heroku.com) - demo #7878.1", "http://glsl.heroku.com/e#7878.1", "demos/demo_7878_1_gl2.xml"])
		store.append (["#8350.0", "GLSL Sandbox (glsl.heroku.com) - demo #8350.0", "http://glsl.heroku.com/e#8350.0", "demos/demo_8350_0_gl2.xml"])
		store.append (["#8628.0", "GLSL Sandbox (glsl.heroku.com) - demo #8628.0", "http://glsl.heroku.com/e#8628.0", "demos/demo_8628_0_gl2.xml"])
		store.append (["#9110.0", "GLSL Sandbox (glsl.heroku.com) - demo #9110.0", "http://glsl.heroku.com/e#9110.0", "demos/demo_9110_0_gl2.xml"])
		store.append (["#9863.0", "GLSL Sandbox (glsl.heroku.com) - demo #9863.0", "http://glsl.heroku.com/e#9863.0", "demos/demo_9863_0_gl2.xml"])
		store.append (["#10557.0", "GLSL Sandbox (glsl.heroku.com) - demo #10557.0", "http://glsl.heroku.com/e#10557.0", "demos/demo_10557_0_gl2.xml"])
		store.append (["#10595.0", "GLSL Sandbox (glsl.heroku.com) - demo #10595.0", "http://glsl.heroku.com/e#10595.0", "demos/demo_10595_0_gl2.xml"])
		store.append (["#11569.0", "GLSL Sandbox (glsl.heroku.com) - demo #11569.0", "http://glsl.heroku.com/e#11569.0", "demos/demo_11569_0_gl2.xml"])
		self.demosComboBox.set_model (store)
		self.demosComboBox.set_active (0)

frontend = FrontendUI ()
Gtk.main ()
